// VCBClass.cpp: implementation of the CVCBClass class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VCBClass.h"
#include "../utils/Gradient.h"



CVCBClass::CVCBClass(RECT rect) : CVCBObject()
{
	// Init members
	m_nObjectType = OBJ_CLASS;
	m_rcObjectRect = rect;
	_tcscpy(m_lpszClassName, _T("CMyClass"));
	_tcscpy(m_lpszClassHeaderFile, _T("MyClass.h"));
	_tcscpy(m_lpszClassImplementationFile, _T("MyClass.cpp"));
	_tcscpy(m_lpszAuthor, _T(""));
	_tcscpy(m_lpszDescription, _T(""));
	m_pVariableList = NULL;
	m_iVariableNumber = 0;
	m_pMethodList = NULL;
	m_iMethodNumber = 0;
	m_pParentList = NULL;
	m_iParentNumber = 0;
	m_bMFCHeaders = FALSE;
	m_bComments = TRUE;

	// Init object fonts
	int fontHeight;
	fontHeight = -MulDiv(10, GetDeviceCaps(GetDC(NULL), LOGPIXELSY), 72);
	m_hClassNameFont = CreateFont(fontHeight, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Times New Roman"));
	fontHeight = -MulDiv(8, GetDeviceCaps(GetDC(NULL), LOGPIXELSY), 72);
	m_hAttributesFont = CreateFont(fontHeight, 0, 0, 0, FW_NORMAL, TRUE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Times New Roman"));
	fontHeight = -MulDiv(8, GetDeviceCaps(GetDC(NULL), LOGPIXELSY), 72);
	m_hMethodsFont = CreateFont(fontHeight, 0, 0, 0, FW_NORMAL, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Times New Roman"));

	// Get screen DC
	HDC hDC = GetDC(NULL);

	// Init memory DC and bitmap
	m_hMemDC = CreateCompatibleDC(hDC);
	m_hMemBitmap = CreateCompatibleBitmap(hDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	m_hOldMemBitmap = (HBITMAP)SelectObject(m_hMemDC, m_hMemBitmap);

	// Init shadow DC and bitmap
	m_hShadowDC = CreateCompatibleDC(hDC);
	m_hShadowBitmap = CreateCompatibleBitmap(hDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	m_hOldShadowBitmap = (HBITMAP)SelectObject(m_hShadowDC, m_hShadowBitmap);

	// Init background DC and bitmap
	m_hBackgroundDC = CreateCompatibleDC(hDC);
	m_hBackgroundBitmap = CreateCompatibleBitmap(hDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	m_hOldBackgroundBitmap = (HBITMAP)SelectObject(m_hBackgroundDC, m_hBackgroundBitmap);

	// Draw object shadow
	DrawShadow();

	// Draw object background
	DrawBackground();

	// Release screen DC
	ReleaseDC(NULL, hDC);
}

CVCBClass::~CVCBClass()
{
	// Delete object fonts
	if (m_hClassNameFont)
	{
		DeleteObject(m_hClassNameFont);
		m_hClassNameFont = NULL;
	}
	if (m_hAttributesFont)
	{
		DeleteObject(m_hAttributesFont);
		m_hAttributesFont = NULL;
	}
	if (m_hMethodsFont)
	{
		DeleteObject(m_hMethodsFont);
		m_hMethodsFont = NULL;
	}

	// Check for valid variable list
	if (m_pVariableList)
	{
		// Destroy variable list
		free(m_pVariableList);
		m_pVariableList = NULL;
		m_iVariableNumber = 0;
	}

	// Check for valid method list
	if (m_pMethodList)
	{
		// Destroy method list
		free(m_pMethodList);
		m_pMethodList = NULL;
		m_iMethodNumber = 0;
	}

	// Check for valid parent list
	if (m_pParentList)
	{
		// Destroy parent list
		free(m_pParentList);
		m_pParentList = NULL;
		m_iParentNumber = 0;
	}
}

void CVCBClass::DrawVCBObject(HDC hDC)
{
	int i;
	CString strText, strAccessLevel;

	// Draw object shadow
	BitBlt(m_hMemDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top, m_hShadowDC, 0, 0, SRCCOPY);

	// Draw object background
	BitBlt(m_hMemDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, m_rcObjectRect.bottom-m_rcObjectRect.top-SHADOW_DISTANCE, m_hBackgroundDC, 0, 0, SRCCOPY);

	int oldBkMode = SetBkMode(m_hMemDC, TRANSPARENT);

	// Draw class name
	COLORREF oldTextColor = SetTextColor(m_hMemDC, RGB(128,0,0));
	HFONT hOldFont = (HFONT)SelectObject(m_hMemDC, m_hClassNameFont);
	RECT classNameRect = {5, 5, m_rcObjectRect.right-m_rcObjectRect.left-10, 20};
	CString strClassName = m_lpszClassName;
	if (m_iParentNumber > 0)
	{
		strClassName.Format(_T("%s : "), m_lpszClassName);
		for (i=0; i<m_iParentNumber; i++)
		{
			strClassName += m_pParentList[i].lpszClassName;
			if (i < m_iParentNumber-1)
				strClassName += _T(", ");
		}
	}
	DrawText(m_hMemDC, strClassName, _tcslen(strClassName), &classNameRect, DT_END_ELLIPSIS | DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_WORD_ELLIPSIS);

	// Draw class members
	SetTextColor(m_hMemDC, RGB(0,0,128));
	SelectObject(m_hMemDC, m_hAttributesFont);
	RECT memberRect = {5, 27, m_rcObjectRect.right-m_rcObjectRect.left-5, 42};
	for (i=0; i<m_iVariableNumber; i++)
	{
		switch (m_pVariableList[i].variableAccessLevel)
		{
			case AL_PUBLIC:
			{
				strAccessLevel = _T("+");
			}
			break;

			case AL_PROTECTED:
			{
				strAccessLevel = _T("*");
			}
			break;

			case AL_PRIVATE:
			{
				strAccessLevel = _T("-");
			}
			break;
		}
		if (m_pVariableList[i].bStatic)
			strText.Format(_T("(s, %s) %s : %s"), strAccessLevel, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableType);
		else
			strText.Format(_T("(%s) %s : %s"), strAccessLevel, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableType);
		DrawText(m_hMemDC, strText, _tcslen(strText), &memberRect, DT_END_ELLIPSIS | DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_WORD_ELLIPSIS);
		OffsetRect(&memberRect, 0, 15);

		// Check for full members list
		if (memberRect.top > 60)
			break;
	}

	// Draw class methods
	SetTextColor(m_hMemDC, RGB(0,128,0));
	SelectObject(m_hMemDC, m_hMethodsFont);
	RECT methodRect = {5, 77, m_rcObjectRect.right-m_rcObjectRect.left-5, 92};
	for (i=0; i<m_iMethodNumber; i++)
	{
		switch (m_pMethodList[i].methodAccessLevel)
		{
			case AL_PUBLIC:
			{
				strAccessLevel = _T("+");
			}
			break;

			case AL_PROTECTED:
			{
				strAccessLevel = _T("*");
			}
			break;

			case AL_PRIVATE:
			{
				strAccessLevel = _T("-");
			}
			break;
		}
		if (m_pMethodList[i].bStatic)
		{
			if (m_pMethodList[i].bVirtual)
				strText.Format(_T("(s, v, %s) %s : %s"), strAccessLevel, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodType);
			else
				strText.Format(_T("(s, %s) %s : %s"), strAccessLevel, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodType);
		}
		else
		{
			if (m_pMethodList[i].bVirtual)
				strText.Format(_T("(v, %s) %s : %s"), strAccessLevel, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodType);
			else
				strText.Format(_T("(%s) %s : %s"), strAccessLevel, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodType);
		}
		DrawText(m_hMemDC, strText, _tcslen(strText), &methodRect, DT_END_ELLIPSIS | DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_WORD_ELLIPSIS);
		OffsetRect(&methodRect, 0, 15);

		// Check for full methods list
		if (methodRect.top > 190)
			break;
	}

	SetTextColor(m_hMemDC, RGB(0,0,128));
	SelectObject(m_hMemDC, hOldFont);
	SetBkMode(m_hMemDC, oldBkMode);

	// Draw object selection
	if (m_bIsSelected)
	{
		LOGBRUSH lb;
		lb.lbStyle = BS_HATCHED;
		lb.lbHatch = HS_FDIAGONAL;
		lb.lbColor = RGB(0,0,128);
		HPEN hSelectionPen = ExtCreatePen(PS_GEOMETRIC, 3, &lb, 0, NULL);
		HPEN hOldPen = (HPEN)SelectObject(m_hMemDC, hSelectionPen);
		HBRUSH hOldBrush = (HBRUSH)SelectObject(m_hMemDC, GetStockObject(HOLLOW_BRUSH));
		Rectangle(m_hMemDC, 1, 1, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, m_rcObjectRect.bottom-m_rcObjectRect.top-SHADOW_DISTANCE);
		SelectObject(m_hMemDC, hOldBrush);
		SelectObject(m_hMemDC, hOldPen);
		DeleteObject(hSelectionPen);
	}

	// Update transparency
	UpdateTransparency(hDC);

	// Blit on screenDC
	BitBlt(hDC, m_rcObjectRect.left, m_rcObjectRect.top, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top, m_hMemDC, 0, 0, SRCCOPY);
}

void CVCBClass::DrawShadow()
{
	// Clear shadow DC background
	HBRUSH hBgBrush = CreateSolidBrush(RGB(255,255,255));
	RECT bgRect = {0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top};
	FillRect(m_hShadowDC, &bgRect, hBgBrush);
	DeleteObject(hBgBrush);

	int shadowDistance = SHADOW_DISTANCE / 2;
	int shadowWidth = abs(m_rcObjectRect.right - m_rcObjectRect.left - shadowDistance);
	int shadowHeight = abs(m_rcObjectRect.bottom - m_rcObjectRect.top - shadowDistance);
	RECT shadowRect = {0, 0, shadowWidth, shadowHeight};
	HBRUSH hShadowBgBrush = CreateSolidBrush(RGB(255,255,255));
	HBRUSH hShadowBrush = CreateSolidBrush(RGB(0,0,0));
	int filterSize = FILTER_SIZE;

	if ((shadowWidth > filterSize) && (shadowHeight > filterSize))
	{
		HDC hScreenDC = GetDC(NULL);
		HDC hShadowDC = CreateCompatibleDC(hScreenDC);
		HBITMAP hShadowBitmap = CreateCompatibleBitmap(hScreenDC, shadowWidth, shadowHeight);
		HBITMAP hOldShadowBitmap = (HBITMAP)SelectObject(hShadowDC, hShadowBitmap);
		ReleaseDC(NULL, hScreenDC);

		// Draw shadow rectangle
		FillRect(hShadowDC, &shadowRect, hShadowBgBrush);
		InflateRect(&shadowRect, -shadowDistance, -shadowDistance);
		FillRect(hShadowDC, &shadowRect, hShadowBrush);

		BITMAP shadowBMP;
		GetObject(hShadowBitmap, sizeof(BITMAP), &shadowBMP);
		int size = shadowBMP.bmWidthBytes * shadowBMP.bmHeight;
		int bpp = shadowBMP.bmBitsPixel / 8;
		UCHAR* pShadowBits = new UCHAR[size];
		UCHAR* pTempShadowBits = new UCHAR[size];
		GetBitmapBits(hShadowBitmap, size, pShadowBits);
		memcpy(pTempShadowBits, pShadowBits, size);
		int red, green, blue;
		int imageOffsetY = 0;
		int imageOffsetX = 0;
		int imageOffset;
		int filterOffsetX;
		int filterOffsetY;
		int filterOffset;
		int filterHalfSize = filterSize / 2;
		BOOL bLeftEdge, bTopEdge, bRightEdge, bBottomEdge;
		for (int i=0; i<shadowBMP.bmHeight; i++)
		{
			imageOffsetX = 0;
			for (int j=0; j<shadowBMP.bmWidth; j++)
			{
				bLeftEdge = (j <= filterSize-1);
				bTopEdge = (i <= filterSize-1);
				bRightEdge =  (j >= shadowBMP.bmWidth-filterSize+1);
				bBottomEdge = (i >= shadowBMP.bmHeight-filterSize+1);

				if (bLeftEdge || bTopEdge || bRightEdge || bBottomEdge)
				{
					red = green = blue = 0;
					for (int k=-filterHalfSize; k<filterHalfSize; k++)
					{
						filterOffsetY = min(max(0, i+k), shadowBMP.bmHeight-1) * shadowBMP.bmWidthBytes;
						for (int l=-filterHalfSize; l<filterHalfSize; l++)
						{
							filterOffsetX = min(max(0, j+l), shadowBMP.bmWidth) * bpp;
							filterOffset = filterOffsetY + filterOffsetX;
							
							blue += pShadowBits[filterOffset];
							green += pShadowBits[filterOffset+1];
							red += pShadowBits[filterOffset+2];

							filterOffsetX += bpp;
						}

						filterOffsetY += shadowBMP.bmWidthBytes;
					}

					imageOffset = imageOffsetY + imageOffsetX;

					if ((imageOffset+2) < size)
					{
						blue = blue >> filterSize;
						green = green >> filterSize;
						red = red >> filterSize;
						pTempShadowBits[imageOffset] = blue;
						pTempShadowBits[imageOffset+1] = green;
						pTempShadowBits[imageOffset+2] = red;
					}
				}

				imageOffsetX += bpp;
			}

			imageOffsetY += shadowBMP.bmWidthBytes;
		}

		SetBitmapBits(hShadowBitmap, size, pTempShadowBits);
		delete pTempShadowBits;
		delete pShadowBits;

		BitBlt(m_hShadowDC, shadowDistance, shadowDistance, shadowWidth, shadowHeight, hShadowDC, 0, 0, SRCCOPY);

		DeleteObject(hShadowBgBrush);
		DeleteObject(hShadowBrush);

		if (hShadowDC)
		{
			SelectObject(hShadowDC, hOldShadowBitmap);
			DeleteDC(hShadowDC);
			DeleteObject(hShadowBitmap);
		}
	}
}

void CVCBClass::DrawBackground()
{
	// Clear object background
	HBRUSH hBgBrush = CreateSolidBrush(RGB(255,255,255));
	RECT bgRect = {0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top};
	FillRect(m_hMemDC, &bgRect, hBgBrush);
	DeleteObject(hBgBrush);

	// Draw object background
	CGradient gradient;
	RECT rect = {bgRect.left, bgRect.top, bgRect.right-3, bgRect.bottom-3};
	gradient.VerticalGradient(m_hBackgroundDC, &rect, RGB(255,255,200), RGB(255,255,255), TRUE, 1.4);
	HPEN hPen = CreatePen(PS_SOLID, 1, RGB(128,0,0));
	HPEN hOldPen = (HPEN)SelectObject(m_hBackgroundDC, hPen);
	HBRUSH hOldBrush = (HBRUSH)SelectObject(m_hBackgroundDC, GetStockObject(HOLLOW_BRUSH));
	Rectangle(m_hBackgroundDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, m_rcObjectRect.bottom-m_rcObjectRect.top-SHADOW_DISTANCE);
	Rectangle(m_hBackgroundDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, 75);
	Rectangle(m_hBackgroundDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left-SHADOW_DISTANCE, 25);
	SelectObject(m_hBackgroundDC, hOldBrush);
	SelectObject(m_hBackgroundDC, hOldPen);
	DeleteObject(hPen);
}

void CVCBClass::UpdateTransparency(HDC hDC)
{
	int i, j;
	int imageOffset, imageOffsetX, imageOffsetY;

	HDC hScreenDC = GetDC(NULL);
	HDC hTempDC = CreateCompatibleDC(hScreenDC);
	HBITMAP hTempBitmap = CreateCompatibleBitmap(hScreenDC, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top);
	HBITMAP hOldTempBitmap = (HBITMAP)SelectObject(hTempDC, hTempBitmap);
	ReleaseDC(NULL, hScreenDC);

	// Blit from screen DC
	BitBlt(hTempDC, 0, 0, m_rcObjectRect.right-m_rcObjectRect.left, m_rcObjectRect.bottom-m_rcObjectRect.top, hDC, m_rcObjectRect.left, m_rcObjectRect.top, SRCCOPY);

	// Get screen data
	BITMAP tempBMP;
	GetObject(hTempBitmap, sizeof(BITMAP), &tempBMP);
	int size = tempBMP.bmWidthBytes * tempBMP.bmHeight;
	int bpp = tempBMP.bmBitsPixel / 8;
	UCHAR* pTempBits = new UCHAR[size];
	GetBitmapBits(hTempBitmap, size, pTempBits);

	// Get memory data
	UCHAR* pMemBits = new UCHAR[size];
	GetBitmapBits(m_hMemBitmap, size, pMemBits);

	// Update shadow transparency
	imageOffsetY = 0;
	imageOffsetX = 0;
	for (i=0; i<tempBMP.bmHeight; i++)
	{
		imageOffsetX = 0;
		for (j=0; j<tempBMP.bmWidth; j++)
		{
			imageOffset = imageOffsetY + imageOffsetX;

			if ((imageOffset+2) < size)
			{
				// Check for full-transparent region
				if ((((i>=tempBMP.bmHeight-SHADOW_DISTANCE) && (i<tempBMP.bmHeight)) && ((j>=0) && (j<SHADOW_DISTANCE))) || 
					(((i>=0) && (i<SHADOW_DISTANCE)) && ((j>=tempBMP.bmWidth-SHADOW_DISTANCE) && (j<tempBMP.bmWidth))))
				{
					pMemBits[imageOffset] = pTempBits[imageOffset];
					pMemBits[imageOffset+1] = pTempBits[imageOffset+1];
					pMemBits[imageOffset+2] = pTempBits[imageOffset+2];
				}
				// Check for semi-transparent region
				else if ((((i>=SHADOW_DISTANCE) && (i<tempBMP.bmHeight)) && ((j>=tempBMP.bmWidth-SHADOW_DISTANCE) && (j<tempBMP.bmWidth))) || 
					(((i>=tempBMP.bmHeight-SHADOW_DISTANCE) && (i<tempBMP.bmHeight)) && ((j>=SHADOW_DISTANCE) && (j<tempBMP.bmWidth))))
				{
					pMemBits[imageOffset] = (UCHAR)max(0, min(255, (pMemBits[imageOffset]*SHADOW_TRANSPARENCY + pTempBits[imageOffset]*(1-SHADOW_TRANSPARENCY))));
					pMemBits[imageOffset+1] = (UCHAR)max(0, min(255, (UCHAR)(pMemBits[imageOffset+1]*SHADOW_TRANSPARENCY + pTempBits[imageOffset+1]*(1-SHADOW_TRANSPARENCY))));
					pMemBits[imageOffset+2] = (UCHAR)max(0, min(255, (UCHAR)(pMemBits[imageOffset+2]*SHADOW_TRANSPARENCY + pTempBits[imageOffset+2]*(1-SHADOW_TRANSPARENCY))));
				}
			}

			imageOffsetX += bpp;
		}

		imageOffsetY += tempBMP.bmWidthBytes;
	}

	SetBitmapBits(m_hMemBitmap, size, pMemBits);
	delete pTempBits;
	delete pMemBits;

	if (hTempDC)
	{
		SelectObject(hTempDC, hOldTempBitmap);
		DeleteDC(hTempDC);
		DeleteObject(hTempBitmap);
	}
}

void CVCBClass::SelectVCBObject(POINT pt)
{
	// Check if object is selected
	m_bIsSelected = PtInRect(&m_rcObjectRect, pt);
}

RECT CVCBClass::GetVCBObjectRect()
{
	// Get object rectangle
	return m_rcObjectRect;
}

void CVCBClass::MoveObject(int offsetX, int offsetY)
{
	// Move object
	m_rcObjectRect.left += offsetX;
	m_rcObjectRect.top += offsetY;
	m_rcObjectRect.right += offsetX;
	m_rcObjectRect.bottom += offsetY;
}

_TCHAR* CVCBClass::GetVCBClassName()
{
	// Get class name
	return m_lpszClassName;
}

void CVCBClass::SetVCBClassName(_TCHAR* lpszClassName)
{
	// Check for valid class name
	if (_tcscmp(lpszClassName, _T("")))
	{
		// Set class name
		_tcscpy(m_lpszClassName, lpszClassName);
	}
}

_TCHAR* CVCBClass::GetVCBClassHeaderFile()
{
	// Get header file
	return m_lpszClassHeaderFile;
}

void CVCBClass::SetVCBClassHeaderFile(_TCHAR* lpszClassHeaderFile)
{
	// Check for valid header file
	if (_tcscmp(lpszClassHeaderFile, _T("")))
	{
		// Set class header file
		_tcscpy(m_lpszClassHeaderFile, lpszClassHeaderFile);
	}
}

_TCHAR* CVCBClass::GetVCBClassImplementationFile()
{
	// Get implementation file
	return m_lpszClassImplementationFile;
}

void CVCBClass::SetVCBClassImplementationFile(_TCHAR* lpszClassImplementationFile)
{
	// Check for valid implementation file
	if (_tcscmp(lpszClassImplementationFile, _T("")))
	{
		// Set class implementation file
		_tcscpy(m_lpszClassImplementationFile, lpszClassImplementationFile);
	}
}

_TCHAR* CVCBClass::GetVCBClassAuthor()
{
	// Get author
	return m_lpszAuthor;
}

void CVCBClass::SetVCBClassAuthor(_TCHAR* lpszAuthor)
{
	// Check for valid author
	if (_tcscmp(lpszAuthor, _T("")))
	{
		// Set author
		_tcscpy(m_lpszAuthor, lpszAuthor);
	}
}

_TCHAR* CVCBClass::GetVCBClassDescription()
{
	// Get description
	return m_lpszDescription;
}

void CVCBClass::SetVCBClassDescription(_TCHAR* lpszDescription)
{
	// Check for valid description
	if (_tcscmp(lpszDescription, _T("")))
	{
		// Set description
		_tcscpy(m_lpszDescription, lpszDescription);
	}
}

int CVCBClass::GetVCBClassVariableNumber()
{
	// Get variable number
	return m_iVariableNumber;
}

void CVCBClass::SetVCBClassVariableNumber(int number)
{
	// Set variable number
	m_iVariableNumber = number;
}

VARIABLE_TYPE* CVCBClass::GetVCBClassVariables()
{
	// Get variable list
	return m_pVariableList;
}

void CVCBClass::SetVCBClassVariables(VARIABLE_TYPE* pVariableList)
{
	// Set variable list
	m_pVariableList = pVariableList;
}

void CVCBClass::GenerateOutputFile()
{
	// Generate class header file
	GenerateHeaderFile();

	// Generate class implementation file
	GenerateImplementationFile();
}

int CVCBClass::GetVCBClassMethodNumber()
{
	// Get method number
	return m_iMethodNumber;
}

void CVCBClass::SetVCBClassMethodNumber(int number)
{
	// Set method number
	m_iMethodNumber = number;
}

METHOD_TYPE* CVCBClass::GetVCBClassMethods()
{
	// Get method list
	return m_pMethodList;
}

void CVCBClass::SetVCBClassMethods(METHOD_TYPE* pMethodList)
{
	// Set method list
	m_pMethodList = pMethodList;
}

int CVCBClass::GetVCBClassParentNumber()
{
	// Get parent number
	return m_iParentNumber;
}

void CVCBClass::SetVCBClassParentNumber(int number)
{
	// Set parent number
	m_iParentNumber = number;
}

PARENT_NODE* CVCBClass::GetVCBClassParents()
{
	// Get parent list
	return m_pParentList;
}

void CVCBClass::SetVCBClassParents(PARENT_NODE* pParentList)
{
	// Set parent list
	m_pParentList = pParentList;
}

BOOL CVCBClass::GetVCBClassMFCHeaders()
{
	// Get MFC headers support
	return m_bMFCHeaders;
}

void CVCBClass::SetVCBClassMFCHeaders(BOOL bMFCHeaders)
{
	// Set MFC headers support
	m_bMFCHeaders = bMFCHeaders;
}

BOOL CVCBClass::GetVCBClassComments()
{
	// Get comments support
	return m_bComments;
}

void CVCBClass::SetVCBClassComments(BOOL bComments)
{
	// Set comments support
	m_bComments = bComments;
}

void CVCBClass::GenerateHeaderFile()
{
	int i;
	CStdioFile outputFile;
	CString strOutput, strClassDefinitionID, strProperty;

	// Create class definition ID
	strClassDefinitionID = m_lpszClassName;
	if (strClassDefinitionID.Left(1) == _T("C"))
		strClassDefinitionID = strClassDefinitionID.Right(strClassDefinitionID.GetLength()-1);
	strClassDefinitionID.MakeUpper();
	strClassDefinitionID += _T("_H");

	// Open output file
	strOutput.Format(_T("output/%s"), m_lpszClassHeaderFile);
	outputFile.Open(strOutput, CFile::modeCreate | CFile::modeWrite);

	// Write author info
	strOutput.Format(_T("// Author : %s\n"), m_lpszAuthor);
	outputFile.WriteString(strOutput);

	// Write description info
	strOutput.Format(_T("// Description : %s\n\n\n"), m_lpszDescription);
	outputFile.WriteString(strOutput);

	// Write start class definitions
	strOutput.Format(_T("#if !defined(%s)\n"), strClassDefinitionID);
	outputFile.WriteString(strOutput);
	strOutput.Format(_T("#define %s\n\n\n"), strClassDefinitionID);
	outputFile.WriteString(strOutput);

	// Write class parents
	CString strParents;
	for (i=0; i<m_iParentNumber; i++)
	{
		switch (m_pParentList[i].classAccessLevel)
		{
			case AL_PUBLIC:
			{
				strParents += _T("public ");
			}
			break;

			case AL_PROTECTED:
			{
				strParents += _T("protected ");
			}
			break;

			case AL_PRIVATE:
			{
				strParents += _T("private ");
			}
			break;
		}
		strParents += m_pParentList[i].lpszClassName;

		if (i < m_iParentNumber-1)
			strParents += _T(", ");

		strOutput.Format(_T("#include \"%s\"\n"), m_pParentList[i].lpszClassHeaderFile);
		outputFile.WriteString(strOutput);
	}

	// Write class header
	if (m_iParentNumber == 0)
		strOutput.Format(_T("class %s\n{\n"), m_lpszClassName);
	else
		strOutput.Format(_T("\n\nclass %s : %s\n{\n"), m_lpszClassName, strParents);
	outputFile.WriteString(strOutput);

	// Write class constructor and destructor
	strOutput.Format(_T("public:\n\t%s();\n\t~%s();\n"), m_lpszClassName, m_lpszClassName);
	outputFile.WriteString(strOutput);

	// Write class properties
	strOutput.Format(_T("\n\npublic:\n\t// %s properties\n"), m_lpszClassName);
	outputFile.WriteString(strOutput);
	for (i=0; i<m_iVariableNumber; i++)
	{
		// Check for class property
		if (m_pVariableList[i].bProperty)
		{
			// Write class property
			strProperty = m_pVariableList[i].lpszPropertyName;
			strProperty.MakeLower();
			strOutput.Format(_T("\t%s Get%s()\t{return %s;}\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszPropertyName, m_pVariableList[i].lpszVariableName);
			outputFile.WriteString(strOutput);
			strOutput.Format(_T("\tvoid Set%s(%s %s)\t{%s = %s;}\n"), m_pVariableList[i].lpszPropertyName, m_pVariableList[i].lpszVariableType, strProperty, m_pVariableList[i].lpszVariableName, strProperty);
			outputFile.WriteString(strOutput);
		}
	}

	// Write class public methods
	strOutput.Format(_T("\n\npublic:\n\t// %s public methods\n"), m_lpszClassName);
	outputFile.WriteString(strOutput);
	for (i=0; i<m_iMethodNumber; i++)
	{
		// Check for public class method
		if (m_pMethodList[i].methodAccessLevel == AL_PUBLIC)
		{
			// Write public class method
			if (m_pMethodList[i].bStatic)
			{
				if (m_pMethodList[i].bVirtual)
				{
					if (m_bComments)
						strOutput.Format(_T("\tstatic virtual %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tstatic virtual %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
				else
				{
					if (m_bComments)
						strOutput.Format(_T("\tstatic %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tstatic %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
			}
			else
			{
				if (m_pMethodList[i].bVirtual)
				{
					if (m_bComments)
						strOutput.Format(_T("\tvirtual %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tvirtual %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
				else
				{
					if (m_bComments)
						strOutput.Format(_T("\t%s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\t%s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
			}
			outputFile.WriteString(strOutput);
		}
	}

	// Write class protected methods
	strOutput.Format(_T("\nprotected:\n\t// %s protected methods\n"), m_lpszClassName);
	outputFile.WriteString(strOutput);
	for (i=0; i<m_iMethodNumber; i++)
	{
		// Check for protected class method
		if (m_pMethodList[i].methodAccessLevel == AL_PROTECTED)
		{
			// Write protected class method
			if (m_pMethodList[i].bStatic)
			{
				if (m_pMethodList[i].bVirtual)
				{
					if (m_bComments)
						strOutput.Format(_T("\tstatic virtual %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tstatic virtual %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
				else
				{
					if (m_bComments)
						strOutput.Format(_T("\tstatic %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tstatic %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
			}
			else
			{
				if (m_pMethodList[i].bVirtual)
				{
					if (m_bComments)
						strOutput.Format(_T("\tvirtual %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tvirtual %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
				else
				{
					if (m_bComments)
						strOutput.Format(_T("\t%s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\t%s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
			}
			outputFile.WriteString(strOutput);
		}
	}

	// Write class private methods
	strOutput.Format(_T("\nprivate:\n\t// %s private methods\n"), m_lpszClassName);
	outputFile.WriteString(strOutput);
	for (i=0; i<m_iMethodNumber; i++)
	{
		// Check for private class method
		if (m_pMethodList[i].methodAccessLevel == AL_PRIVATE)
		{
			// Write private class method
			if (m_pMethodList[i].bStatic)
			{
				if (m_pMethodList[i].bVirtual)
				{
					if (m_bComments)
						strOutput.Format(_T("\tstatic virtual %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tstatic virtual %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
				else
				{
					if (m_bComments)
						strOutput.Format(_T("\tstatic %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tstatic %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
			}
			else
			{
				if (m_pMethodList[i].bVirtual)
				{
					if (m_bComments)
						strOutput.Format(_T("\tvirtual %s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\tvirtual %s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
				else
				{
					if (m_bComments)
						strOutput.Format(_T("\t%s %s;\t\t\t// %s\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName, m_pMethodList[i].lpszMethodComment);
					else
						strOutput.Format(_T("\t%s %s;\t\t\n"), m_pMethodList[i].lpszMethodType, m_pMethodList[i].lpszMethodName);
				}
			}
			outputFile.WriteString(strOutput);
		}
	}

	// Write class public members
	strOutput.Format(_T("\n\npublic:\n\t// %s public members\n"), m_lpszClassName);
	outputFile.WriteString(strOutput);
	for (i=0; i<m_iVariableNumber; i++)
	{
		// Check for public class member
		if (m_pVariableList[i].variableAccessLevel == AL_PUBLIC)
		{
			// Write public class member
			if (m_pVariableList[i].bStatic)
			{
				if (m_bComments)
					strOutput.Format(_T("\tstatic %s %s;\t\t\t// %s\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableComment);
				else
					strOutput.Format(_T("\tstatic %s %s;\t\t\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName);
			}
			else
			{
				if (m_bComments)
					strOutput.Format(_T("\t%s %s;\t\t\t// %s\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableComment);
				else
					strOutput.Format(_T("\t%s %s;\t\t\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName);
			}
			outputFile.WriteString(strOutput);
		}
	}

	// Write class protected members
	strOutput.Format(_T("\nprotected:\n\t// %s protected members\n"), m_lpszClassName);
	outputFile.WriteString(strOutput);
	for (i=0; i<m_iVariableNumber; i++)
	{
		// Check for protected class member
		if (m_pVariableList[i].variableAccessLevel == AL_PROTECTED)
		{
			// Write protected class member
			if (m_pVariableList[i].bStatic)
			{
				if (m_bComments)
					strOutput.Format(_T("\tstatic %s %s;\t\t\t// %s\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableComment);
				else
					strOutput.Format(_T("\tstatic %s %s;\t\t\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName);
			}
			else
			{
				if (m_bComments)
					strOutput.Format(_T("\t%s %s;\t\t\t// %s\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableComment);
				else
					strOutput.Format(_T("\t%s %s;\t\t\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName);
			}
			outputFile.WriteString(strOutput);
		}
	}

	// Write class private members
	strOutput.Format(_T("\nprivate:\n\t// %s private members\n"), m_lpszClassName);
	outputFile.WriteString(strOutput);
	for (i=0; i<m_iVariableNumber; i++)
	{
		// Check for private class member
		if (m_pVariableList[i].variableAccessLevel == AL_PRIVATE)
		{
			// Write private class member
			if (m_pVariableList[i].bStatic)
			{
				if (m_bComments)
					strOutput.Format(_T("\tstatic %s %s;\t\t\t// %s\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableComment);
				else
					strOutput.Format(_T("\tstatic %s %s;\t\t\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName);
			}
			else
			{
				if (m_bComments)
					strOutput.Format(_T("\t%s %s;\t\t\t// %s\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName, m_pVariableList[i].lpszVariableComment);
				else
					strOutput.Format(_T("\t%s %s;\t\t\n"), m_pVariableList[i].lpszVariableType, m_pVariableList[i].lpszVariableName);
			}
			outputFile.WriteString(strOutput);
		}
	}

	// Write class footer
	strOutput.Format(_T("\n};\n\n"));
	outputFile.WriteString(strOutput);

	// Write end class definitions
	strOutput.Format(_T("#endif // !defined(%s)\n"), strClassDefinitionID);
	outputFile.WriteString(strOutput);

	// Close output file
	outputFile.Close();
}

void CVCBClass::GenerateImplementationFile()
{
	int i;
	CStdioFile outputFile;
	CString strOutput;

	// Open output file
	strOutput.Format(_T("output/%s"), m_lpszClassImplementationFile);
	outputFile.Open(strOutput, CFile::modeCreate | CFile::modeWrite);

	// Write MFC precompiled headers support
	if (m_bMFCHeaders)
	{
		strOutput.Format(_T("#include \"stdafx.h\"\n"));
		outputFile.WriteString(strOutput);
	}

	// Write class includes
	strOutput.Format(_T("#include \"%s\"\n\n"), m_lpszClassHeaderFile);
	outputFile.WriteString(strOutput);

	// Write class constructur and destructor
	strOutput.Format(_T("\n%s::%s()\n{\n\t// Add your code here\n}\n"), m_lpszClassName,m_lpszClassName);
	outputFile.WriteString(strOutput);
	strOutput.Format(_T("\n%s::~%s()\n{\n\t// Add your code here\n}\n"), m_lpszClassName,m_lpszClassName);
	outputFile.WriteString(strOutput);

	// Write class methods
	for (i=0; i<m_iMethodNumber; i++)
	{
		// Write class method
		strOutput.Format(_T("\n%s %s::%s\n{\n\t// Add your code here\n}\n"), m_pMethodList[i].lpszMethodType, m_lpszClassName, m_pMethodList[i].lpszMethodName);
		outputFile.WriteString(strOutput);
	}

	// Close output file
	outputFile.Close();
}

void CVCBClass::SaveObject(CArchive& ar)
{
	int i;

	// Save CVCBClass object
	ar << m_nObjectType;
	ar << m_dwObjectID;
	ar << m_rcObjectRect;
	ar << (CString)m_lpszClassName;
	ar << (CString)m_lpszClassHeaderFile;
	ar << (CString)m_lpszClassImplementationFile;
	ar << (CString)m_lpszAuthor;
	ar << (CString)m_lpszDescription;
	ar << m_iVariableNumber;
	for (i=0; i<m_iVariableNumber; i++)
	{
		ar << (CString)m_pVariableList[i].lpszVariableType;
		ar << (CString)m_pVariableList[i].lpszVariableName;
		ar << (CString)m_pVariableList[i].lpszVariableComment;
		ar << m_pVariableList[i].bProperty;
		ar << (CString)m_pVariableList[i].lpszPropertyName;
		ar << m_pVariableList[i].bStatic;
		ar << m_pVariableList[i].variableAccessLevel;
	}
	ar << m_iMethodNumber;
	for (i=0; i<m_iMethodNumber; i++)
	{
		ar << (CString)m_pMethodList[i].lpszMethodType;
		ar << (CString)m_pMethodList[i].lpszMethodName;
		ar << (CString)m_pMethodList[i].lpszMethodComment;
		ar << m_pMethodList[i].bVirtual;
		ar << m_pMethodList[i].bStatic;
		ar << m_pMethodList[i].methodAccessLevel;
	}
	ar << m_iParentNumber;
	for (i=0; i<m_iParentNumber; i++)
	{
		ar << (CString)m_pParentList[i].lpszClassName;
		ar << m_pParentList[i].classAccessLevel;
		ar << (CString)m_pParentList[i].lpszClassHeaderFile;
	}
	ar << m_bMFCHeaders;
	ar << m_bComments;
}
